library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity main is
    Port ( quartz_I : in std_logic;
           clk_I    : in std_logic;
           start_I  : in std_logic;
           err_I    : in std_logic_vector(2 downto 0);
           qclk_O   : out std_logic;
           heater_O : out std_logic;
           fan_O    : out std_logic;
           led_rt_O : out std_logic;
           led_gn_O : out std_logic;
           beeper_O : out std_logic
         );
end main;

architecture Behavioral of main is
--Zwischenspeicher
signal qprecnt  : std_logic_vector(13 downto 0);
signal trst     : std_logic;
signal tprecnt  : std_logic_vector(4 downto 0);
signal tqcnt    : std_logic_vector(4 downto 0);
signal stop     : std_logic;
signal error    : std_logic;
signal tcnt9    : std_logic;
signal tcnt4    : std_logic;


component precnt
   PORT(
         takt : in std_logic;
         q    : out std_logic_vector(13 downto 0)
        );
end component;

component tcnt
   PORT(
         takt  : in std_logic;
         reset : in std_logic;
         load  : in std_logic;
         d     : in std_logic_vector(4 downto 0);
         q     : out std_logic_vector(4 downto 0)
        );
end component;


for all : precnt use entity work.precnt(Behavioral);
for all : tcnt   use entity work.tcnt(Behavioral);

begin
   precounter : precnt port map(quartz_I,qprecnt);
   timer      : tcnt port map(clk_I,trst,start_I,tprecnt,tqcnt);

   -- static logic
   qclk_O   <= qprecnt(13);  -- clkout 2Hz

   tprecnt  <= "10100";  -- 20

   error    <= err_I(0) AND err_I(1) AND err_I(2);
   led_rt_O <= error;
   led_gn_O <= NOT(error);

   trst     <= error OR (NOT(start_I) AND stop);
   
   fan_O    <= NOT(stop);  
   heater_O <= tcnt9;

   beeper_O <= tcnt4 AND NOT(tcnt9) AND qprecnt(13) AND qprecnt(4);

   -- timer cnt
   count: PROCESS (tqcnt)
   BEGIN
     IF tqcnt = "00000" THEN stop <= '1';
                        ELSE stop <= '0';
     END IF;
     
     IF tqcnt > "01001" THEN tcnt9 <= '1';
                        ELSE tcnt9 <= '0';
     END IF;
     
     IF tqcnt > "00100" THEN tcnt4 <= '1';
                        ELSE tcnt4 <= '0';
     END IF;

   END PROCESS count;

end Behavioral;
